"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs");
var path = require("path");
var index_1 = require("typedoc/dist/lib/models/reflections/index");
var index_2 = require("typedoc/dist/lib/models/reflections/index");
var UrlMapping_1 = require("typedoc/dist/lib/output/models/UrlMapping");
var DefaultTheme_1 = require("typedoc/dist/lib/output/themes/DefaultTheme");
var service_1 = require("./service");
var MarkdownTheme = (function (_super) {
    __extends(MarkdownTheme, _super);
    function MarkdownTheme(renderer, basePath, options) {
        var _this = _super.call(this, renderer, basePath) || this;
        renderer.removeComponent('assets');
        renderer.removeComponent('javascript-index');
        renderer.removeComponent('navigation');
        renderer.removeComponent('toc');
        renderer.removeComponent('pretty-print');
        service_1.ThemeService.options = options;
        service_1.ThemeService.resources = _this.resources;
        return _this;
    }
    MarkdownTheme.buildUrls = function (reflection, urls) {
        var mapping = DefaultTheme_1.DefaultTheme.getMapping(reflection);
        if (mapping) {
            var url = [mapping.directory, MarkdownTheme.getUrl(reflection) + '.md'].join('/');
            urls.push(new UrlMapping_1.UrlMapping(url, reflection, mapping.template));
            reflection.url = url;
            reflection.hasOwnDocument = true;
            for (var key in reflection.children) {
                if (reflection.children.hasOwnProperty(key)) {
                    var child = reflection.children[key];
                    if (mapping.isLeaf) {
                        MarkdownTheme.applyAnchorUrl(child, reflection);
                    }
                    else {
                        MarkdownTheme.buildUrls(child, urls);
                    }
                }
            }
        }
        else {
            MarkdownTheme.applyAnchorUrl(reflection, reflection.parent);
        }
        return urls;
    };
    MarkdownTheme.applyAnchorUrl = function (reflection, container) {
        var options = service_1.ThemeService.getOptions();
        var anchor = DefaultTheme_1.DefaultTheme.getUrl(reflection, container, '.');
        if (reflection['isStatic']) {
            anchor = 'static-' + anchor;
        }
        var anchorRef = '';
        switch (reflection.kind) {
            case index_2.ReflectionKind.ExternalModule:
                anchorRef = "external-module-" + service_1.ThemeService.getAnchorRef(reflection.name) + "-";
                break;
            case index_2.ReflectionKind.Class:
                anchorRef = "class-" + service_1.ThemeService.getAnchorRef(reflection.name);
                break;
            case index_2.ReflectionKind.Interface:
                anchorRef = "interface-" + service_1.ThemeService.getAnchorRef(reflection.name);
                break;
            case index_2.ReflectionKind.Module:
                anchorRef = "module-" + service_1.ThemeService.getAnchorRef(reflection.name);
            case index_2.ReflectionKind.Enum:
                if (reflection.parent.kind === 0 || reflection.parent.kind === index_2.ReflectionKind.ExternalModule) {
                    anchorRef = "module-" + service_1.ThemeService.getAnchorRef(reflection.name);
                }
                else {
                    anchorRef = "enumeration-" + service_1.ThemeService.getAnchorRef(reflection.name);
                }
                break;
            default:
                if (options.mdFlavour === 'bitbucket') {
                    var anchorPrefix_1 = '';
                    if (reflection.kind === index_2.ReflectionKind.ObjectLiteral) {
                        anchorPrefix_1 += 'object-literal-';
                    }
                    reflection.flags.forEach(function (flag) {
                        anchorPrefix_1 += flag + "-";
                    });
                    var prefixRef = service_1.ThemeService.getAnchorRef(anchorPrefix_1);
                    var reflectionRef = service_1.ThemeService.getAnchorRef(reflection.name);
                    anchorRef = "markdown-header-" + prefixRef + reflectionRef;
                }
                else {
                    anchorRef = anchor;
                }
        }
        reflection.url = (container.url !== undefined ? container.url : '') + '#' + anchorRef;
        reflection.anchor = anchor;
        reflection.hasOwnDocument = false;
        reflection.traverse(function (child) {
            if (child instanceof index_1.DeclarationReflection) {
                MarkdownTheme.applyAnchorUrl(child, container);
            }
        });
    };
    MarkdownTheme.prototype.isOutputDirectory = function (outPath) {
        var files = fs.readdirSync(outPath);
        return fs.existsSync(path.join(outPath, 'README.md')) || (files.length === 1 && path.extname(files[0]) === '.md');
    };
    MarkdownTheme.prototype.getUrls = function (project) {
        var urls = [];
        var entryPoint = this.getEntryPoint(project);
        service_1.ThemeService.projectName = entryPoint.name;
        var additionalContext = {
            displayReadme: this.application.options.getValue('readme') !== 'none',
            hideBreadcrumbs: true,
            isIndex: true,
        };
        var context = Object.assign(entryPoint, additionalContext);
        urls.push(new UrlMapping_1.UrlMapping('README.md', context, 'reflection.hbs'));
        if (entryPoint.children) {
            entryPoint.children.forEach(function (child) {
                MarkdownTheme.buildUrls(child, urls);
            });
        }
        return urls;
    };
    return MarkdownTheme;
}(DefaultTheme_1.DefaultTheme));
exports.MarkdownTheme = MarkdownTheme;
